
/*
 * ctype.h - macros for character tests and conversions
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): hybrid-2k
 * started: 2003-06-26
 * changed: 2003-06-27
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#ifndef _CTYPE_H_
#define _CTYPE_H_

/*
 * these defines should act just like the usual ctype.h functions
 * and according to ANSI C and C99 standards. anyway, in some cases
 * they might be uncompatible with useual ctype.h functions, and if
 * you find any of those cases please fix the problem or at least
 * report to the maintainer of this file.
 */

/* alphanumeric: [a-z], [A-Z], [0-9] */
#define isalnum(c) \
  (((c)>='a' && (c)<='z') || ((c)>='A' && (c)<='Z') || ((c)>='0' && (c)<='9'))

/* alphabetic: [a-z], [A-Z] */
#define isalpha(c) \
  (((c)>='a' && (c)<='z') || ((c)>='A' && (c)<='Z'))

/* control characters */
#define iscntrl(c) \
  ((c)>=0 && (c<=31))

/* digit: [0-9] */
#define isdigit(c) \
  ((c)>='0' && (c)<='9')

/* ASCII 7bit characters */
#define isascii(c) \
  ((c)>=0 && (c<=127))

/* graphic characters */
#define isgraph(c) \
  ((c)>=128 && (c<=255)) /* is this right? */

/* lower-case letter: [a-z] */
#define islower(c) \
  ((c)>='a' && (c)<='z')

/* upper-case letter: [A-Z] */
#define isupper(c) \
  ((c)>='A' && (c)<='Z')

/* printable character */
#define isprint(c) \
  ((c)>=32 && (c<=255))

/* punctuation mark: . , ; : ! ? */
#define ispunct(c) \
  ((c)=='.' || (c)==',' || (c)==';' || (c)==':' || (c)=='!' || (c)=='?')

/* blank: sp, \t */
#define isblank(c) \
  ((c)==' ' || (c)=='\t')

/* white space: sp, \f, \n, \r, \t, \v */
#define isspace(c)
  ((c)==' ' || (c)=='\f' || (c)=='\n' || (c)=='\r' || (c)=='\t' || (c)=='\v')

/* hex digit [0-9], [a-f], [A-F] */
#define isxdigit(c) \
  ((c)>='0' && (c)<='9')

/* convert to 7bit ASCII */
#define toascii(c) \
  ((c) & 0x7F)

/* convert to lower-case */
#define tolower(c) \
  ((c) - 'A'+'a')

/* convert to upper-case */
#define toupper(c) \
  ((c) - 'a'+'A')

#endif /* ! _CTYPE_H_ */
