
/*
 * syscalls.h - future kernel system calls (header)
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): hybrid-2k
 * started: 2003-03-02
 * changed: 2003-06-21
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#ifndef _SYSCALLS_H_
#define _SYSCALLS_H_

#include <sys/types.h>

#define POSIX_SYSCALLS		/* Unix, Linux, FreeBSD, NetBSD, OpenBSD, BeOS, AtheOS, etc. */
#define LINUX_SYSCALLS		/* Linux specific syscalls */
#define LINUX_FILE64_SYSCALLS	/* syscalls to handle 64bit files */

#define STDIN  0
#define STDOUT 1
#define STDERR 2

/**********************************************************************/

#ifdef LINUX_SYSCALLS
#define POSIX_SYSCALLS
#endif

#ifdef POSIX_SYSCALLS
#define SC_exit        1	/* terminate the current process */
#define SC_fork        2	/* create child process */
#define SC_read        3	/* read from file descriptor */
#define SC_write       4	/* write to file descriptor */
#define SC_open        5	/* open / create file or device */
#define SC_close       6	/* close a file descriptor */
#define SC_link        9	/* assign new name to a file */
#define SC_unlink     10	/* del name & file it refers to */
#define SC_execve     11	/* execute program */
#define SC_chdir      12	/* change working directory */
#define SC_mknod      14	/* create dir, special, ordinary file */
#define SC_chmod      15	/* change permissions of a file */
#define SC_lseek      19	/* set read/write file offset */
#define SC_mount      21	/* mount and unmount filesystems */
#define SC_setuid     23	/* set user identity */
#define SC_getuid     24	/* get user identity */
#define SC_alarm      27	/* set alarm clock for delivery of signal */
#define SC_utime      30	/* chg access and/or modification times */
#define SC_access     33	/* check user's permissions for a file */
#define SC_sync       36	/* commit buffer cache to disk */
#define SC_kill       37	/* send signal to a process */
#define SC_rename     38	/* change the name or location of a file */
#define SC_mkdir      39	/* create directory */
#define SC_rmdir      40	/* delete (an empty) directory */
#define SC_dup        41	/* duplicate a file descriptor */
#define SC_pipe       42	/* create pipe */
#define SC_brk        45	/* change data segment size */
#define SC_setgid     46	/* set group identity */
#define SC_getgid     47	/* get group identity */
#define SC_signal     48	/* ANSI C signal handling */
#define SC_ioctl      54	/* re ioctl assignations , control device */
#define SC_setpgid    57	/* set/get process group */
#define SC_chroot     61	/* change root directory */
#define SC_dup2       63	/* replace 2nd w/ duplicate of 1st fd */
#define SC_getppid    64	/* get process identification */
#define SC_getpgrp    65	/* set/get process group */
#define SC_setsid     66	/* crea session, set process group ID */
#define SC_sigaction  67	/* POSIX signal handling functions */
#define SC_getgroups  80	/* get list of supplementary group IDs */
#define SC_symlink    83	/* make new name for a file */
#define SC_readlink   85	/* read value of a symbolic link */
#define SC_reboot     88	/* reboot or enable/disable Ctrl+Alt+Del */
#define SC_stat      106	/* get file status */
#define SC_fstat     108	/* get file status */
#define SC_uname     122	/* get name & info about current kernel */
#define SC_nanosleep 162	/* pause execution for specified */
#endif

#ifdef LINUX_SYSCALLS
#define SC_time          13	/* get time in seconds */
#define SC_stime         25	/* set time in seconds */
#define SC_gettimeofday  78	/* get time */
#define SC_settimeofday  79	/* set time */
#define SC_ftruncate     93	/* truncate file to specified length  */
#endif

#ifdef LINUX_FILE64_SYSCALLS
#define SC_llseek       140	/* set read/write file offset */
#define SC_ftruncate64  194	/* truncate file to specified length  */
#define SC_stat64       195	/* get file status */
#define SC_fstat64      197	/* get file status */
#endif

/**********************************************************************/

#ifdef POSIX_SYSCALLS
long sys_exit(int error_code);
int sys_fork(struct pt_regs regs);
ssize_t sys_read(unsigned int fd, char *buf, size_t count);
ssize_t sys_write(unsigned int fd, const char *buf, size_t count);
long sys_open(const char *filename, int flags, int mode);
long sys_close(unsigned int fd);
long sys_link(const char *oldname, const char *newname);
long sys_unlink(const char *pathname);
int sys_execve(struct pt_regs regs);
long sys_chdir(const char *filename);
long sys_mknod(const char *filename, int mode, dev_t dev);
long sys_chmod(const char *filename, mode_t mode);
off_t sys_lseek(unsigned int fd, off_t offset, unsigned int origin);
int sys_mount(char *dev_name, char *dir_name, char *type, unsigned long new_flags, void *data);
int sys_setuid(uid_t uid);
int sys_getuid(void), unsigned int sys_alarm(unsigned int seconds);
int sys_utime(char *filename, struct utimbuf *times);
int sys_access(const char *filename, int modeint sys_sync(void);
int sys_sync(void);
void sys_kill(int pid, int sig);
int sys_pipe(unsigned long *fildes);
int sys_setpgid(pid_t pid, pid_t pgid);
int sys_rename(const char *oldname, const char *newname);
int sys_mkdir(const char *pathname, int mode);
int sys_rmdir(const char *pathname);
int sys_dup(unsigned int fildes);
unsigned long sys_brk(unsigned long brk);
int sys_setgid(gid_t gid);
int sys_getgid(void);
int sys_signal(int sig, __sighandler_t handler);
int sys_ioctl(unsigned int fd, unsigned int cmd, unsigned long arg);
int sys_chroot(const char *filename);
int sys_dup2(unsigned int oldfd, unsigned int newfd);
int sys_getppid(void);
int sys_getpgrp(void);
int sys_setsid(void);
int sys_sigaction(int signum, const struct sigaction *action, struct sigaction *oldaction);
int sys_getgroups(int gidsetsize, gid_t * grouplist);
int sys_symlink(const char *oldname, const char *newname);
int sys_readlink(const char *path, char *buf, int bufsiz);
int sys_reboot(int magic1, int magic2, int cmd, void *arg);
int sys_stat(char *filename, struct __old_kernel_stat *statbuf);
int sys_fstat(unsigned int fd, struct __old_kernel_stat *statbuf); int sys_nanosleep(struct timespec *rqtp, struct timespec *rmtp);
#endif

#ifdef LINUX_SYSCALLS
long sys_time(int *tloc)
long sys_stime(int *tptr)
long sys_gettimeofday(struct timeval *tv, struct timezone *tz)
long sys_settimeofday(struct timeval *tv, struct timezone *tz)
int sys_ftruncate(unsigned int fd, unsigned long length)
#endif

#ifdef LINUX_FILE64_SYSCALLS
off64_t sys_llseek(int fd, offset_t offset, int whence);
int sys_ftruncate64(int fildes, off64_t length);
int sys_stat64(const char *path, struct stat64 *buf); int sys_fstat64(int fd, struct stat64 *buf);
#endif

#endif /* ! _SYSCALLS_H_ */
