
/*
 * boot.S - kernel elf boot code
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): hybrid-2k
 * started: 2003.03.01
 * changed: 2003-06-21
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#define ASM 1
#include "mboot.h"
#define STACK_SIZE 16*1024 /* 16kb */
	
	.text

	.globl _start

_start:
	jmp	multiboot_entry


multiboot_header:
	.align	4
	.long	MULTIBOOT_HEADER_MAGIC
	.long	MULTIBOOT_HEADER_FLAGS
	.long	-(MULTIBOOT_HEADER_MAGIC + MULTIBOOT_HEADER_FLAGS)
	.long	multiboot_header
	.long	_start
	.long	_edata
	.long	_end
	.long	multiboot_entry	

multiboot_entry:
	/* set kernel stack pointer and flags register */
	movl	$(stack + STACK_SIZE), %esp
	pushl	$0
	popf

	/* save magic value and mbi struct for later use */
	movl	%eax, magic
	movl	%ebx, mbi
	
	/* Clear the screen before writing anything to it */
	call	EXT_C(vid_clear_screen)

	/* print call message */
	pushl	kernel_main
	pushl	$call_msg
	call	EXT_C(printk)

	/* call kernel main function from kernel.c */
	pushl	mbi
	pushl	magic
	call	EXT_C(kernel_main)

	/* print halt message */
	pushl	%eax
	pushl	$halt_msg
	call	EXT_C(printk)


	/* halt cpu and loop if it fails */
halt:	hlt
	jmp	halt

	/* mesage strings to be displyed */
call_msg:	.asciz	"Calling kernel main function at 0x%x..\n"
halt_msg:	.asciz	"Halted kernel with code %d!\n"

	/* temporary variables */
	.comm	magic, 4
	.comm	mbi, 4

	/* kernel stack */
	.comm	stack, STACK_SIZE
