
/*
 * kernel.c - LusitanOS kernel main function
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): hybrid-2k, mahound
 * started: 2003-03-09
 * changed: 2003-06-21
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include "mboot.h"
#include "kstdio.h"
#include "cpuid.h"
#include "video.h"


#define LUSITANOS_VERSION "LusitanOS v0.03"

#define MBI_FLAG(mbi,bit) ((mbi->flags) & (1 << (bit)))
#define MBI_BOOT_DEVICE_DRIVE(mbi) (((mbi->boot_device) >> 24) & 0xFF)
#define MBI_BOOT_DEVICE_PART(mbi) (((mbi->boot_device) >> 16) & 0xFF)
#define MBI_UNUSED_PART 0xFF



int kernel_main(unsigned long magic, multiboot_info_t * mbi)
{

	int i;

	vid_init();


	/* Check if a valid magic value was passed by grub to the kernel */
	if (magic != MULTIBOOT_BOOTLOADER_MAGIC) {
		printk("Invalid magic number: 0x%x\n", magic);
		return -1;
	}

	/* Just show some narcissist message */
	printk("%s kernel loaded!\n\n", LUSITANOS_VERSION);

	/* Raw Multiboot Info Flags dword */
	printk("Multiboot Info Flags: 0x%x\n", mbi->flags);

	/* Show Comand Line used to invoke the kernel */
	if (MBI_FLAG(mbi, 2)) {
		printk("Kernel Command Line: %s\n", (char *) mbi->cmdline);
	}

	/* Display System Phycical Memory */
	if (MBI_FLAG(mbi, 0)) {
		printk("Memory\n");
		printk("\tPhysical: %dMB\n", (mbi->mem_upper / 1024) + 1);
		printk("\tLower: %dKB\n", mbi->mem_lower);
		printk("\tUpper: %dKB\n", mbi->mem_upper);
	}

	/* Display GRUB-like Boot Device */
	if (MBI_FLAG(mbi, 1)) {
		printk("Boot Device\n");
		printk("\tCode: 0x%x\n", mbi->boot_device);
		printk("\tName: ");
		if (MBI_BOOT_DEVICE_DRIVE(mbi) < 0x80) {
			printk("fd%d\n", MBI_BOOT_DEVICE_DRIVE(mbi));
		} else {
			printk("hd%d\n", MBI_BOOT_DEVICE_DRIVE(mbi) - 0x80);
			if (MBI_BOOT_DEVICE_PART(mbi) != MBI_UNUSED_PART) {
				printk(",%d\n", MBI_BOOT_DEVICE_PART(mbi));
			}
		}
	}

	/* Check for CPUID and use it to display CPU info */
	printk("CPU Info\n");
	if (hasCPUID()) {
		int cpufamily = cpuid();
	} else {
		printk("\t486 or older model\n");
		printk("\tCPUID instruction not supported\n");
	}

	/* Return from kernel_main */



	printk("\nWelcome to ");

	vid_set_color(0x02);

	printk("Lusitan");

	vid_set_color(0x04);

	printk("OS");

	vid_set_color(0x07);

	printk("...\n\n");



	return 69;
}


