
/*
 * kstdio.c - kernel basic input/output functions
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): mahound
 * started: 2003-03-16
 * changed: 2003-06-21
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include "kstdio.h"
#include "video.h"


/* Convert an integer to a string. */
void itoa(char *buf, int base, int d)
{
	char *p = buf;
	char *p1, *p2;
	unsigned long ud = d;
	int divisor = 10;

	if (base == 'd' && d < 0) {
		*p++ = '-';
		buf++;
		ud = -d;
	}
	else {
		if (base == 'x') divisor = 16;
	}

	do {
		int remainder = ud % divisor;
		*p++ = (remainder < 10) ? remainder + '0' : remainder + 'a' - 10;
	}
	while (ud /= divisor);

	*p = 0;

	p1 = buf;
	p2 = p - 1;
	while (p1 < p2) {
		char tmp = *p1;
		*p1 = *p2;
		*p2 = tmp;
		p1++;
		p2--;
	}
}

/* Put a character on the screen. */
void putchar(int c)
{
	vid_write_char(c);
}

/* Format a string and print it on the screen. */
void printk(const char *format, ...)
{
	char **arg = (char **) &format;
	int c;
	char buf[20];
	arg++;

	while ((c = *format++) != 0) {
		if (c != '%') {
			putchar(c);
		}
		else {
			char *p;
			c = *format++;
			switch (c) {

			case 'd':
			case 'u':
			case 'x':
				itoa(buf, c, *((int *) arg++));
				p = buf;
				goto string;
				break;
			case 's':
				p = *arg++;
				if (!p) p = "(null)";
				string:
				while (*p) putchar(*p++);
				break;
			default:
				putchar(*((int *) arg++));
				break;
			}
		}
	}

	vid_show_buffer();
}
