
/*
 * video.c - direct video output functions
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): mahound
 * started: 2003-06-19
 * changed: 2003-06-21
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include "video.h"

#define NUM_LINES		25
#define NUM_COLS		80
#define BUFFER_LINES 		50
#define BUFFER_COLS 		80
#define VID_SIZE_MEM		NUM_LINES*NUM_COLS
#define BUFFER_SIZE		BUFFER_LINES*BUFFER_COLS
#define TAB_SIZE		8

unsigned short* vga = (unsigned short*) 0xB8000;

struct
{
	unsigned short buffer[BUFFER_LINES][BUFFER_COLS];
	unsigned short x_txt;
	unsigned short y_txt;

	unsigned short x_cur;
	unsigned short y_cur;

	unsigned short x_cam;
	unsigned short y_cam;

	unsigned char color;

} text_s;

void set_video(long mode)
{
	__asm__
	("int $0x10"
	:
	:"a" (mode)
	);
}

void fill_ptr(unsigned short* mem, int size, unsigned short chr)
{
	int i;
	for (i=0;i<size;++i)
	{
		mem[i] = chr;
	}
}

void move_cursor(unsigned short pos)
{
	__asm__
	("movw $0x3D4, %%dx\n"
	"movb $0xF, %%al\n"
	"outb %%al, %%dx\n"
	"incw %%dx\n"
	"movb %%bl, %%al\n"
	"outb %%al, %%dx\n"
	"decw %%dx\n"
	"movb $0xE, %%al\n"
	"outb %%al, %%dx\n"
	"incw %%dx\n"
	"movb %%bh, %%al\n"
	"outb %%al, %%dx\n"
	"nop"
	:
	:"b" (pos)
	);
}

void add_line()
{
	int i, j;
	for (i=0;(i<(BUFFER_LINES-1));++i)
	{
		for (j=0;j<BUFFER_COLS;++j)
		{
			text_s.buffer[i][j] = text_s.buffer[i+1][j];
		}
	}
	for (i=0;i<BUFFER_COLS;++i)
	{
		text_s.buffer[BUFFER_LINES-1][i] = 0x0720;
	}
}

void check_cursor()
{
	if (text_s.x_cur>=BUFFER_COLS)
	{
		text_s.x_cur = 0;
		text_s.y_cur++;
	}

	if (text_s.y_cur>=BUFFER_LINES)
	{
		text_s.y_cur = BUFFER_LINES-1;
		add_line();
	}
}

void check_text()
{
	if (text_s.x_txt>BUFFER_COLS)
	{
		text_s.x_txt = 0;
		text_s.y_txt++;
	}

	if (text_s.y_txt>=BUFFER_LINES)
	{
		text_s.y_txt = BUFFER_LINES-1;
	}
}

void update_camera()
{
	if (text_s.y_cur>=(text_s.y_cam+NUM_LINES))
	{
		text_s.y_cam = text_s.y_cur - NUM_LINES + 1;
	}

	if (text_s.y_cur<text_s.y_cam)
	{
		text_s.y_cam = text_s.y_cur;
	}

	move_cursor((text_s.y_cur-text_s.y_cam)*80+text_s.x_cur);

}

void put_cursor(unsigned short line, unsigned short column)
{
	text_s.y_cur = line;
	text_s.x_cur = column;
}

void put_text(unsigned short line, unsigned short column)
{
	text_s.x_txt = column;
	text_s.y_txt = line;
}

void vid_init()
{
	int i, j;

	text_s.x_cur = 0;
	text_s.y_cur = 0;
	text_s.x_cam = 0;
	text_s.y_cam = 0;
	text_s.x_txt = 0;
	text_s.y_txt = 0;
	text_s.color = 0x07;

	for (i=0;i<BUFFER_LINES;++i)
	{
		for (j=0;j<BUFFER_COLS;++j)
		{
			text_s.buffer[i][j] = 0x0720;
		}
	}

}

void vid_set_cursor(unsigned short line, unsigned short column)
{
	put_cursor(line, column);
	check_cursor();
	put_text(line, column);

	update_camera();
}

void vid_update_cursor()
{
	vid_set_cursor(text_s.y_cur, text_s.x_cur );
}

void vid_write_char(int chr)
{
	int i, spaces, next;

	if (chr=='\n')
	{
		text_s.x_txt = 0;
		text_s.y_txt++;
		text_s.x_cur = 0;
		text_s.y_cur++;

		check_cursor();
		check_text();
		//check_cam();



		vid_update_cursor();

		return;

	}



	if (chr=='\r')

	{

		text_s.x_txt = 0;

		text_s.x_cur = 0;



		check_cursor();

		check_text();



		vid_update_cursor();

		return;

	}



	if (chr=='\t')

	{

		next = text_s.x_txt / 8 + 1;

		if (next>9)

		{

			return;

		}

		spaces = next*8 - text_s.x_txt;



		for (i=0;i<spaces;++i)

		{

			vid_write_char(' ');

		}



		return;

	}



	text_s.buffer[text_s.y_txt][text_s.x_txt] = chr | (text_s.color << 8);



	text_s.x_cur++;

	text_s.x_txt++;



	check_cursor();

	check_text();



	vid_update_cursor();

}



void vid_write_char_at(char chr, short line, short column)

{

	put_cursor(line, column);

	vid_write_char(chr);

}



void vid_set_color(unsigned char color)

{

	text_s.color = color;

}



void vid_show_buffer()

{

	int i, j;



	for (i=0;i<NUM_LINES;++i)

	{

		for (j=0;j<NUM_COLS;++j)

		{

			vga[i*NUM_COLS+j] = text_s.buffer[i+text_s.y_cam][j+text_s.x_cam];

		}

	}

}



void vid_clear_screen()

{

	vid_set_cursor(0,0);

	fill_ptr(vga, VID_SIZE_MEM, 0x0720);

}

