
/*
 * string.c - ANSI C string functions
 * Copyright (c) 2003, The LusitanOS Project. All rights reserved.
 *
 * maintainer(s): mahound, hybrid-2k, fozi
 * started: 2003-04-04
 * changed: 2003-06-25
 *
 * This program is free software, and you can redistribute it and/or modify it
 * under the terms of the LSD (Lusitan Source Distribution) License, which are
 * described in the following text.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files, to deal them without
 * restriction, including without limitation the rights to use, copy, modify,
 * merge, publish, distribute, sublicense, and/or sell copies of it, and to
 * permit persons to whom it is furnished to do so, subject to the following
 * conditions:
 *  - The above copyright notice and this permission notice shall be included
 *    in all copies or portions of this software.
 *  - The source code and documentation of this software shall be included
 *    along with all binary distributions and any other kind of product that
 *    uses this software or substantial portions from it.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 *
 */

#include "string.h"

void memcpy(void *dst, const void *src, size_t length)
{
	size_t dwords = length / 4;
	size_t bytes = length % 4;

	while (dwords--) {
		*(((int *) dst)++) = *(((int *) src)++);
	}

	while (bytes--) {
		*(((char *) dst)++) = *(((char *) src)++);
	}
}

void memset(void *dst, char value, size_t length)
{
	size_t dwords = length / 4;
	size_t bytes = length % 4;
	size_t value_expanded = (size_t) (value & 0x00FF | (value & 0x00FF) << 8 | (value & 0x00FF) << 16 | (value & 0x00FF) << 24);

	while (dwords--) {
		*(((int *) dst)++) = value_expanded;
	}

	while (bytes--) {
		*(((char *) dst)++) = value;
	}
}

void memcpy(void *dst, const void *src, size_t length)
{
	size_t dwords = length / 4;

	while (dwords--) {
		*(((long int *) dst)++) = *(((long int *) src)++);
	}

	if ((length -= dwords * 4) != 0) {
		while (length--) {
			*(((char *) dst)++) = *(((char *) src)++)
		}
	}
}

void memset(void *dst, char value, size_t length)
{
	while (length--) {
		*(((char *) dst)++) = value;
	}
}

int strcmp(const char *str1, const char *str2)
{
	int not_equal = 0;
	while (*str1) {
		if (!(*str2)) {
			return 1;
		} else if (*str1 == *str2) {
			str1++;
			str2++;
		} else {
			not_equal++;
			break;
		}
	}
	if (not_equal || ((unsigned char) (*str2))) {
		return -1;
	} else {
		return 0;
	}
}

int strncmp(const char *str1, const char *str2, size_t size)
{
	int not_equal = 0;
	while ((*str1) && (size)) {
		if (!(*str2)) {
			return 1;
		} else if (*str1 == *str2) {
			str1++;
			str2++;
			size--;
		} else {
			not_equal++;
			break;
		}
	}
	if (not_equal || ((size != 0) && ((unsigned char) (*str2)))) {
		return -1;
	} else {
		return 0;
	}
}

char *strncpy(char *dest, const char *orig, size_t size)
{
	char *pdest = (char *) dest;
	while (size) {
		if (*orig) {
			*pdest = *orig;
			orig++;
		} else {
			*pdest = '\0';
		}
		pdest++;
		size--;
	}
	return dest;
}

char *strcpy(char *dest, const char *orig)
{
	char c = 1;
	unsigned int n = 0;

	while (c) {
		c = orig[n];
		dest[n] = c;
		n++;
	}
	n++;
	dest[n] = '\0';
	return dest;
}

char *strcat(char *dest, const char *origin)
{
	char *pdest = dest;
	while (*pdest) {
		pdest++;
	}
	strcpy(pdest, origin);
	return dest;
}

char *strncat(char *dest, const char *origin, size_t size)
{
	char *pdest = dest;
	while (*pdest) {
		pdest++;
	}
	while (size) {
		if (*origin) {
			*pdest = *origin;
		} else {
			break;
		}
		origin++;
		pdest++;
		size--;
	}
	*pdest = '\0';
	return dest;
}

size_t strlen(const char *string)
{
	unsigned int n = 0;
	char c = 1;
	while (c) {
		c = string[n];
		n++;
	}
	return (size_t) (n - 1);
}

char *strchr(const char *string, char chr)
{
	char *c = (char *) string;

	while (*c) {
		if (*c == chr) {
			return c;
		}
		c++;
	}
	return '\0';
}

char *strrchr(const char *string, char chr)
{
	char *c = (char *) string;

	while (*c) {
		c++;
	}

	c--;
	while (c != string) {
		if (*c == chr) {
			return c;
		} else {
			c--;
		}
	}
	return '\0';
}
